@@ -18,13 +18,13 @@ package pqarrow
 
 import (
 	"encoding/base64"
-	"errors"
 	"fmt"
 	"math"
 	"strconv"
 
 	"github.com/apache/arrow-go/v18/arrow"
 	"github.com/apache/arrow-go/v18/arrow/decimal128"
+	"github.com/apache/arrow-go/v18/arrow/extensions"
 	"github.com/apache/arrow-go/v18/arrow/flight"
 	"github.com/apache/arrow-go/v18/arrow/memory"
 	"github.com/apache/arrow-go/v18/parquet"
@@ -241,20 +241,30 @@ func repFromNullable(isnullable bool) parquet.Repetition {
 	return parquet.Repetitions.Required
 }
 
-func variantToNode(t *variantExtensionType, field arrow.Field, props *parquet.WriterProperties, arrProps ArrowWriterProperties) (schema.Node, error) {
-	metadataNode, err := fieldToNode("metadata", t.Metadata(), props, arrProps)
+func variantToNode(t *extensions.VariantType, field arrow.Field, props *parquet.WriterProperties, arrProps ArrowWriterProperties) (schema.Node, error) {
+	fields := make(schema.FieldList, 2, 3)
+	var err error
+
+	fields[0], err = fieldToNode("metadata", t.Metadata(), props, arrProps)
 	if err != nil {
 		return nil, err
 	}
 
-	valueNode, err := fieldToNode("value", t.Value(), props, arrProps)
+	fields[1], err = fieldToNode("value", t.Value(), props, arrProps)
 	if err != nil {
 		return nil, err
 	}
 
+	if typed := t.TypedValue(); typed.Type != nil {
+		typedValue, err := fieldToNode("typed_value", typed, props, arrProps)
+		if err != nil {
+			return nil, err
+		}
+		fields = append(fields, typedValue)
+	}
+
 	return schema.NewGroupNodeLogical(field.Name, repFromNullable(field.Nullable),
-		schema.FieldList{metadataNode, valueNode}, schema.VariantLogicalType{},
-		fieldIDFromMeta(field.Metadata))
+		fields, schema.VariantLogicalType{}, fieldIDFromMeta(field.Metadata))
 }
 
 func structToNode(field arrow.Field, props *parquet.WriterProperties, arrprops ArrowWriterProperties) (schema.Node, error) {
@@ -326,7 +336,7 @@ func fieldToNode(name string, field arrow.Field, props *parquet.WriterProperties
 	case arrow.EXTENSION:
 		extType := field.Type.(arrow.ExtensionType)
 		if extType.ExtensionName() == "parquet.variant" {
-			return variantToNode(extType.(*variantExtensionType), field, props, arrprops)
+			return variantToNode(extType.(*extensions.VariantType), field, props, arrprops)
 		}
 	}
 
@@ -853,19 +863,45 @@ func mapToSchemaField(n *schema.GroupNode, currentLevels file.LevelInfo, ctx *sc
 	return nil
 }
 
-func variantToSchemaField(n *schema.GroupNode, currentLevels file.LevelInfo, ctx *schemaTree, parent, out *SchemaField) error {
-	// this is for unshredded variants. shredded variants may have more fields
-	if n.NumFields() != 2 {
-		return errors.New("VARIANT group must have exactly 2 children")
+func variantToSchemaField(n *schema.GroupNode, currentLevels file.LevelInfo, ctx *schemaTree, _, out *SchemaField) error {
+	switch n.NumFields() {
+	case 2, 3:
+	default:
+		return fmt.Errorf("VARIANT group must have exactly 2 or 3 children, not %d", n.NumFields())
 	}
 
+	if n.RepetitionType() == parquet.Repetitions.Repeated {
+		// list of variants
+		out.Children = make([]SchemaField, 1)
+		repeatedAncestorDef := currentLevels.IncrementRepeated()
+		if err := groupToStructField(n, currentLevels, ctx, &out.Children[0]); err != nil {
+			return err
+		}
+
+		storageType := out.Children[0].Field.Type
+		elemType, err := extensions.NewVariantType(storageType)
+		if err != nil {
+			return err
+		}
+
+		out.Children[0].Field.Type = elemType
+		out.Field = &arrow.Field{Name: n.Name(), Type: arrow.ListOfField(*out.Children[0].Field), Nullable: true,
+			Metadata: createFieldMeta(int(n.FieldID()))}
+		ctx.LinkParent(&out.Children[0], out)
+		out.LevelInfo = currentLevels
+		out.LevelInfo.RepeatedAncestorDefLevel = repeatedAncestorDef
+		return nil
+	}
+
+	currentLevels.Increment(n)
+
 	var err error
 	if err = groupToStructField(n, currentLevels, ctx, out); err != nil {
 		return err
 	}
 
 	storageType := out.Field.Type
-	out.Field.Type, err = newVariantType(storageType)
+	out.Field.Type, err = extensions.NewVariantType(storageType)
 	return err
 }
 
@@ -1011,19 +1047,19 @@ func getNestedFactory(origin, inferred arrow.DataType) func(fieldList []arrow.Fi
 		switch origin.ID() {
 		case arrow.LIST:
 			return func(list []arrow.Field) arrow.DataType {
-				return arrow.ListOf(list[0].Type)
+				return arrow.ListOfField(list[0])
 			}
 		case arrow.FIXED_SIZE_LIST:
 			sz := origin.(*arrow.FixedSizeListType).Len()
 			return func(list []arrow.Field) arrow.DataType {
-				return arrow.FixedSizeListOf(sz, list[0].Type)
+				return arrow.FixedSizeListOfField(sz, list[0])
 			}
 		}
 	case arrow.MAP:
 		if origin.ID() == arrow.MAP {
 			return func(list []arrow.Field) arrow.DataType {
 				valType := list[0].Type.(*arrow.StructType)
-				return arrow.MapOf(valType.Field(0).Type, valType.Field(1).Type)
+				return arrow.MapOfFields(valType.Field(0), valType.Field(1))
 			}
 		}
 	}
